#include <Fl/Fl.h>
#include "TargaImage.h"
#include "ImageWidget.h"
#include <Fl/Fl_Window.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>


// Static variables have scope only in the current file.
static ImageWidget  *image_widget = NULL; // The widget that displays the image
static TargaImage   *current_image = NULL;// The current image object


static int
Load_Function(char *filename)
{
    TargaImage	*new_image;

    fprintf(stderr, "Loading %s...\n", filename);

    new_image = TargaImage::Load_Image(filename);
    if ( new_image == NULL )
    {
	    fprintf(stderr, "Error loading image from file %s\n", filename);
	    return 0;
    }

    // Free the memory from the current image
    delete current_image;

    // Replace the current image
    current_image = new_image;
    image_widget->Set_Image(current_image);

    return 1;
}


// Argument processing callback. Does nothing at this point.
static int
Arg_Callback(int argc, char *argv[], int &i)
{
    return 0;
}

int
main(int argc, char *argv[])
{
    Fl_Window	window(400, 400, "CS559 Assignment 2");
    int		script_arg;

    // Do argument processing. At the end of this, script_arg contains
    // the first non-switch argument, which if not 0 or argc is the
    // location of the script file name in the argument list.
    Fl::args(argc, argv, script_arg, Arg_Callback);
    if ( script_arg == 0 )
    {
	    fprintf(stderr, "Error: Unrecognised argument.\n");
	    return 1;
    }

    Fl::visual(FL_RGB);

    window.begin();

    // We use a separate image containing widget so that other widgets, like
    // buttins, can be added to the main window. This creates that image
    // display widget, and then sets it as the thing to be resized when the
    // window is resized. This allows us to resize thing properly when the
    // image size changes.
    image_widget = new ImageWidget(0, 0, 400, 400, "Image");
    window.add(*image_widget);

    window.end();

    if ( script_arg < argc )
    {
	    if ( ! Load_Function(argv[script_arg]) )
	    {
	        return 1;
	    }
    }

    window.show(argc, argv);

    return Fl::run();
}


